var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ChainId } from "../common";
import { Slippages } from "./slippages";
import { Zero } from "@ethersproject/constants";
import { Tokens } from "../tokens";
export var BridgeUtils;
(function (BridgeUtils) {
    const ETH_CHAINS = [
        ChainId.OPTIMISM,
        ChainId.BOBA,
        ChainId.MOONBEAM,
        ChainId.ARBITRUM,
        ChainId.AVALANCHE,
        ChainId.HARMONY,
    ];
    BridgeUtils.isL2ETHChain = (chainId) => ETH_CHAINS.includes(chainId);
    BridgeUtils.DepositIfChainTokens = [
        { chainId: ChainId.FANTOM, tokens: [Tokens.JUMP], depositEth: false },
        { chainId: ChainId.POLYGON, tokens: [Tokens.NFD], depositEth: false },
        { chainId: ChainId.ARBITRUM, tokens: [Tokens.GMX], depositEth: false },
        { chainId: ChainId.MOONRIVER, tokens: [Tokens.SOLAR], depositEth: false },
        { chainId: ChainId.AVALANCHE, tokens: [Tokens.AVAX, Tokens.WAVAX], altChainId: ChainId.MOONBEAM, depositEth: true },
        { chainId: ChainId.MOONRIVER, tokens: [Tokens.MOVR, Tokens.WMOVR], altChainId: ChainId.MOONBEAM, depositEth: true },
    ];
    BridgeUtils.getBridgeTxArgs = () => ({
        slippageCustom: null,
        slippageSelected: Slippages.OneTenth,
        infiniteApproval: true,
        transactionDeadline: BridgeUtils.getTimeMinutesFromNow(10),
        bridgeTransactionDeadline: BridgeUtils.getTimeMinutesFromNow(60 * 24)
    });
    function getSlippages(amountFrom, amountTo) {
        const { slippageSelected, transactionDeadline, bridgeTransactionDeadline } = BridgeUtils.getBridgeTxArgs();
        const selectedGasArgs = slippageSelected, twoTenthGasArgs = Slippages.TwoTenth, quarterGasArgs = Slippages.Quarter;
        const minToSwapOrigin = Slippages.subtractSlippage(amountFrom, selectedGasArgs), minToSwapDest = Slippages.subtractSlippage(amountTo, selectedGasArgs), minToSwapDestFromOrigin = Slippages.subtractSlippage(minToSwapDest, selectedGasArgs);
        const minToSwapOriginMediumSlippage = Slippages.subtractSlippage(amountFrom, twoTenthGasArgs), minToSwapDestMediumSlippage = Slippages.subtractSlippage(amountTo, twoTenthGasArgs), minToSwapDestFromOriginMediumSlippage = Slippages.subtractSlippage(minToSwapDestMediumSlippage, twoTenthGasArgs);
        const minToSwapOriginHighSlippage = Slippages.subtractSlippage(amountFrom, quarterGasArgs), minToSwapDestHighSlippage = Slippages.subtractSlippage(amountTo, quarterGasArgs), minToSwapDestFromOriginHighSlippage = Slippages.subtractSlippage(minToSwapDestHighSlippage, quarterGasArgs);
        return {
            slippageSelected,
            transactionDeadline,
            bridgeTransactionDeadline,
            minToSwapOrigin,
            minToSwapDest,
            minToSwapDestFromOrigin,
            minToSwapOriginMediumSlippage,
            minToSwapDestMediumSlippage,
            minToSwapDestFromOriginMediumSlippage,
            minToSwapOriginHighSlippage,
            minToSwapDestHighSlippage,
            minToSwapDestFromOriginHighSlippage,
        };
    }
    BridgeUtils.getSlippages = getSlippages;
    BridgeUtils.subBigNumSafe = (a, b) => a.gt(b) ? a.sub(b) : Zero;
    BridgeUtils.getTimeMinutesFromNow = (minutesFromNow) => Math.round((new Date().getTime() / 1000) + 60 * minutesFromNow);
    BridgeUtils.makeEasyParams = (args, chainId, t) => [args.addressTo, args.chainIdTo, t.address(chainId), args.amountFrom];
    BridgeUtils.makeEasySubParams = (args, chainId, t) => {
        let x = BridgeUtils.makeEasyParams(args, chainId, t);
        return [x[0], x[1], x[2]];
    };
    BridgeUtils.depositETHParams = (args) => [args.addressTo, args.chainIdTo, args.amountFrom];
    function calculateSwapL2Zap(zapBridge, intermediateToken, tokenIndexFrom, tokenIndexTo, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if ([
                Tokens.WAVAX.address(ChainId.AVALANCHE),
                Tokens.WAVAX.address(ChainId.MOONBEAM),
                Tokens.WMOVR.address(ChainId.MOONBEAM),
                Tokens.WMOVR.address(ChainId.MOONRIVER)
            ].includes(intermediateToken)) {
                console.log(intermediateToken, tokenIndexFrom, tokenIndexTo, amount.toString());
            }
            return zapBridge.calculateSwap(intermediateToken, tokenIndexFrom, tokenIndexTo, amount);
        });
    }
    BridgeUtils.calculateSwapL2Zap = calculateSwapL2Zap;
})(BridgeUtils || (BridgeUtils = {}));
