var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { BigNumber } from "@ethersproject/bignumber";
import { ERC20Factory } from "../contracts";
import { newProviderForNetwork } from "../rpcproviders";
import { executePopulatedTransaction, rejectPromise, } from "../common/utils";
import { GasUtils } from "./gasutils";
export const MAX_APPROVAL_AMOUNT = BigNumber.from("0xffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff");
export var ERC20;
(function (ERC20_1) {
    class ERC20 {
        constructor(args) {
            this.approve = (args, signer) => __awaiter(this, void 0, void 0, function* () { return executePopulatedTransaction(this.buildApproveTransaction(args), signer); });
            this.buildApproveTransaction = (args) => __awaiter(this, void 0, void 0, function* () {
                let { spender, amount } = args;
                amount = amount !== null && amount !== void 0 ? amount : MAX_APPROVAL_AMOUNT;
                return this.instance.populateTransaction.approve(spender, amount)
                    .then((txn) => {
                    let { maxPriorityFee, gasPrice, approveGasLimit } = GasUtils.makeGasParams(this.chainId);
                    if (maxPriorityFee) {
                        txn.maxPriorityFeePerGas = maxPriorityFee;
                    }
                    if (gasPrice) {
                        txn.gasPrice = gasPrice;
                    }
                    if (approveGasLimit) {
                        txn.gasLimit = approveGasLimit;
                    }
                    return txn;
                })
                    .catch(rejectPromise);
            });
            this.balanceOf = (address) => __awaiter(this, void 0, void 0, function* () { return this.instance.balanceOf(address); });
            this.allowanceOf = (owner, spender) => __awaiter(this, void 0, void 0, function* () { return this.instance.allowance(owner, spender); });
            this.address = args.tokenAddress;
            this.chainId = args.chainId;
            this.provider = newProviderForNetwork(this.chainId);
            this.instance = ERC20Factory.connect(this.address, this.provider);
        }
    }
    ERC20_1.approve = (approveArgs, tokenParams, signer) => __awaiter(this, void 0, void 0, function* () { return new ERC20(tokenParams).approve(approveArgs, signer); });
    ERC20_1.buildApproveTransaction = (approveArgs, tokenParams) => __awaiter(this, void 0, void 0, function* () { return new ERC20(tokenParams).buildApproveTransaction(approveArgs); });
    ERC20_1.balanceOf = (address, tokenParams) => __awaiter(this, void 0, void 0, function* () { return new ERC20(tokenParams).balanceOf(address); });
    ERC20_1.allowanceOf = (owner, spender, tokenParams) => __awaiter(this, void 0, void 0, function* () { return new ERC20(tokenParams).allowanceOf(owner, spender); });
})(ERC20 || (ERC20 = {}));
